@@ -7,7 +7,7 @@ use constant WIN32   => $^O eq 'MSWin32';
 use constant CYGWIN  => $^O eq 'cygwin';
 use constant NETWARE => $^O eq 'NetWare';
 use constant WINFU   => WIN32 || CYGWIN || NETWARE;
-use constant COLOR   => ! $ENV{APACHE_TEST_NO_COLOR} && -t STDOUT;
+use constant COLOR   => ($ENV{APACHE_TEST_COLOR} && -t STDOUT) ? 1 : 0;
 
 use constant DEFAULT_PORT => 8529;
 
@@ -142,6 +142,9 @@ sub new {
             require "$_/apache_test_config.pm";
             $thaw = 'apache_test_config'->new;
             delete $thaw->{save};
+            #incase class that generated the config was
+            #something else, which we can't be sure how to load
+            bless $thaw, 'Apache::TestConfig';
         };
     };
 
@@ -723,6 +726,19 @@ sub writefile {
     close $fh;
 }
 
+sub perlscript_header {
+    return <<'EOF';
+
+use strict;
+use warnings FATAL => 'all';
+
+use FindBin;
+use lib map "$FindBin::Bin/$_",
+        qw(../Apache-Test/lib ../lib ../../lib);
+
+EOF
+}
+
 # gen + write executable perl script file
 sub write_perlscript {
     my($self, $file, $content) = @_;
@@ -1223,13 +1239,40 @@ sub error_log {
 
 #utils
 
-#duplicating small bits of Apache::Build so we dont require it
+#For Win32 systems, stores the extensions used for executable files
+
+my @path_ext = ();
+
+if (WIN32) {
+    if ($ENV{PATHEXT}) {
+        push @path_ext, split ';', $ENV{PATHEXT};
+    }
+    else {
+        #Win9X: doesn't have PATHEXT
+        push @path_ext, qw(com exe bat);
+    }
+}
+
 sub which {
-    foreach (map { catfile $_, $_[0] } File::Spec->path) {
-        return $_ if -x;
+    my $program = shift;
+
+    return undef unless $program;
+
+    my @results = ();
+
+    for my $base (map { catfile($_, $program) } File::Spec->path()) {
+        if ($ENV{HOME} and not WIN32) {
+            # only works on Unix, but that's normal:
+            # on Win32 the shell doesn't have special treatment of '~'
+            $base =~ s/~/$ENV{HOME}/o;
+        }
+
+        return $base if -x $base;
+
         if (WIN32) {
-            my $exe = "$_.exe";
-            return $exe if -x $exe;
+            for my $ext (@path_ext) {
+                return "$base.$ext" if -x "$base.$ext";
+            }
         }
     }
 }
