@@ -61,7 +61,7 @@ sub fixup {
     #make sure we use an absolute path to perl
     #else Test::Harness uses the perl in our PATH
     #which might not be the one we want
-    $^X = $Config{perlpath} if $^X eq 'perl';
+    $^X = $Config{perlpath} unless -e $^X;
 }
 
 sub new {
@@ -778,20 +778,12 @@ sub generate_script {
         $content = "\%Apache::TestConfig::Argv = qw(@Apache::TestMM::Argv);\n";
     }
 
-    $content .= <<'EOM';
-use strict;
-use warnings FATAL => 'all';
+    my $header = Apache::TestConfig->perlscript_header;
 
-use FindBin;
-use lib "$FindBin::Bin/../Apache-Test/lib";
-
-use Apache::TestRun ();
-
-Apache::TestRun->new->run(@ARGV);
-EOM
+    $content .= join "\n",
+      $header, "use $class ();", "$class->new->run(\@ARGV);";
 
     Apache::Test::config()->write_perlscript($file, $content);
-
 }
 
 # in idiomatic perl functions return 1 on success 0 on
