@@ -17,7 +17,8 @@ our $VERSION = '1.99';
 
 use Apache::compat ();
 
-use Apache::Response;
+use Apache::Response ();
+use Apache::RequestRec ();
 use Apache::Log;
 use Apache::Const -compile => qw(:common &OPT_EXECCGI);
 use File::Spec::Functions ();
@@ -42,10 +43,11 @@ use constant D_NOISE   => 8;
 # httpd.conf with:
 #   PerlSetVar ModPerl::RegistryCooker::DEBUG 4
 use Apache::ServerUtil ();
-use constant DEBUG =>
-    defined Apache->server->dir_config('ModPerl::RegistryCooker::DEBUG')
-        ? Apache->server->dir_config('ModPerl::RegistryCooker::DEBUG')
-        : D_NONE;
+use constant DEBUG => 0;
+#XXX: below currently crashes the server on win32
+#    defined Apache->server->dir_config('ModPerl::RegistryCooker::DEBUG')
+#        ? Apache->server->dir_config('ModPerl::RegistryCooker::DEBUG')
+#        : D_NONE;
 
 #########################################################################
 # object's array index's access constants
@@ -168,9 +170,9 @@ sub run {
     my $rc = Apache::OK;
     my $cv = \&{"$package\::handler"};
 
-    { # run the code if $r->seqno, preserve warnings setup when it's done
+    { # run the code and preserve warnings setup when it's done
         no warnings;
-        eval { $rc = &{$cv}($r, @_) } if $r->seqno;
+        eval { $rc = &{$cv}($r, @_) };
         $o->[STATUS] = $rc;
         ModPerl::Global::special_list_call(END => $package);
     }
