@@ -9,16 +9,21 @@ use lib qw(Apache-Test/lib);
 use Config;
 use Cwd ();
 use File::Spec ();
+use File::Basename;
 use ExtUtils::Embed ();
 use ModPerl::Code ();
 use ModPerl::BuildOptions ();
 use Apache::TestTrace;
+use Apache::TestConfig ();
 
 use constant REQUIRE_ITHREADS => grep { $^O eq $_ } qw(MSWin32);
 use constant HAS_ITHREADS =>
     $Config{useithreads} && ($Config{useithreads} eq 'define');
 
-use constant is_win32 => $^O eq 'MSWin32';
+use constant DARWIN => $^O eq 'darwin';
+use constant WIN32 => $^O eq 'MSWin32';
+use constant MSVC  => WIN32() && ($Config{cc} eq 'cl');
+
 use constant IS_MOD_PERL_BUILD => grep { -e "$_/lib/mod_perl.pm" } qw(. ..);
 
 our $VERSION = '0.01';
@@ -47,6 +52,34 @@ my %apxs_query = (
     PREFIX     => '',
 );
 
+sub ap_prefix_invalid {
+    my $self = shift;
+
+    my $prefix = $self->{MP_AP_PREFIX};
+
+    unless (-d $prefix) {
+        return "$prefix: No such file or directory";
+    }
+
+    my $include_dir = $self->apxs(-q => 'INCLUDEDIR');
+
+    unless (-e $include_dir) {
+        return "include/ directory not found in $prefix";
+    }
+
+    return '';
+}
+
+sub ap_prefix_is_source_tree {
+    my $self = shift;
+
+    return unless exists $self->{MP_AP_PREFIX};
+
+    my $prefix = $self->{MP_AP_PREFIX};
+
+    -d $prefix and -e "$prefix/CHANGES";
+}
+
 sub apxs {
     my $self = shift;
 
@@ -72,7 +105,7 @@ sub apxs {
         #these extra tries are for things built outside of mod_perl
         #e.g. libapreq
         push @trys,
-        which('apxs'),
+        Apache::TestConfig::which('apxs'),
         '/usr/local/apache/bin/apxs';
     }
 
@@ -111,12 +144,6 @@ sub apxs_cflags {
     $cflags;
 }
 
-sub which {
-    foreach (map { File::Spec->catfile($_, $_[0]) } File::Spec->path) {
-	return $_ if -x;
-    }
-}
-
 #--- Perl Config stuff ---
 
 my @Xlib = qw(/usr/X11/lib /usr/X11R6/lib);
@@ -140,10 +167,17 @@ sub gtop_ldopts {
 sub ldopts {
     my($self) = @_;
 
+    my $config = tied %Config;
+    my $ldflags = $config->{ldflags};
+
+    $config->{ldflags} = '' if WIN32; #same as lddlflags
+
     my $ldopts = ExtUtils::Embed::ldopts();
     chomp $ldopts;
 
-    if ($^O eq 'hpux' and $Config{ld} eq 'ld') {
+    my $ld = $self->perl_config('ld');
+
+    if ($^O eq 'hpux' and $ld eq 'ld') {
         while ($ldopts =~ s/-Wl,(\S+)/$1/) {
             my $cp = $1;
             (my $repl = $cp) =~ s/,/ /g;
@@ -155,6 +189,8 @@ sub ldopts {
         $ldopts .= $self->gtop_ldopts;
     }
 
+    $config->{ldflags} = $ldflags; #reset
+
     $ldopts;
 }
 
@@ -180,7 +216,10 @@ sub ap_ccopts {
 
     if ($self->{MP_DEBUG}) {
         $self->{MP_TRACE} = 1;
-        $ccopts .= " -g -DMP_DEBUG";
+        my $win32_flags = MSVC  ? '-Od -MD -Zi' : '';
+        my $debug_flags = WIN32 ? $win32_flags : '-g';
+        $ccopts .= " $debug_flags" unless $Config{optimize} =~ /$debug_flags/;
+        $ccopts .= ' -DMP_DEBUG';
     }
 
     if ($self->{MP_CCOPTS}) {
@@ -195,13 +234,23 @@ sub ap_ccopts {
 }
 
 sub perl_ccopts {
-    my $cflags = shift->strip_lfs(" $Config{ccflags} ");
+    my $self = shift;
+
+    my $cflags = $self->strip_lfs(" $Config{ccflags} ");
 
     my $fixup = \&{"ccopts_$^O"};
     if (defined &$fixup) {
         $fixup->(\$cflags);
     }
 
+    if ($self->{MP_DEBUG}) {
+        my $optim = $Config{optimize};
+
+        unless ($optim =~ /-DDEBUGGING/) {
+            $cflags =~ s/$optim//;
+       }
+    }
+
     $cflags;
 }
 
@@ -221,10 +270,49 @@ sub ccopts {
     $cflags;
 }
 
+sub perl_config_optimize {
+    my($self, $val) = @_;
+
+    $val ||= $Config{optimize};
+
+    if ($self->{MP_DEBUG}) {
+        return ' ' unless $val =~ /-DDEBUGGING/;
+    }
+
+    $val;
+}
+
+sub perl_config_ld {
+    my($self, $val) = @_;
+
+    $val ||= $Config{ld};
+
+    basename $val; #bleedperl hpux value is /usr/bin/ld !
+}
+
+sub perl_config_lddlflags {
+    my($self, $val) = @_;
+
+    if ($self->{MP_DEBUG}) {
+        if (MSVC) {
+            $val =~ s/-release/-debug/;
+        }
+    }
+
+    $val;
+}
+
 sub perl_config {
     my($self, $key) = @_;
 
-    return $Config{$key} ? $Config{$key} : '';
+    my $val = $Config{$key} || '';
+
+    my $method = \&{"perl_config_$key"};
+    if (defined &$method) {
+        return $method->($self, $val);
+    }
+
+    return $val;
 }
 
 sub find_in_inc {
@@ -376,7 +464,7 @@ sub DESTROY {}
 my %default_files = (
     'build_config' => 'lib/Apache/BuildConfig.pm',
     'ldopts' => 'src/modules/perl/ldopts',
-    'makefile' => 'src/modules/perl/Makefile',
+    'makefile' => 'src/modules/perl/Makefile.modperl',
 );
 
 sub clean_files {
@@ -463,10 +551,7 @@ sub rebuild {
 
 #--- attribute access ---
 
-sub is_dynamic {
-    my $self = shift;
-    $self->{MP_USE_DSO} || $self->{MP_USE_APXS};
-}
+sub is_dynamic { shift->{MP_USE_DSO} }
 
 sub default_dir {
     my $build = shift->build_config;
@@ -489,7 +574,7 @@ sub dir {
 
     return $self->{dir} if $self->{dir};
 
-    if(IS_MOD_PERL_BUILD) {
+    if (IS_MOD_PERL_BUILD) {
         my $build = $self->build_config;
 
         if ($dir = $build->{'dir'}) {
@@ -499,6 +584,8 @@ sub dir {
         }
     }
 
+    $dir ||= $self->{MP_AP_PREFIX};
+
 # we not longer install Apache headers, so dont bother looking in @INC
 # might end up finding 1.x headers anyhow
 #    unless ($dir and -d $dir) {
@@ -551,7 +638,7 @@ sub ap_includedir  {
 sub mmn_eq {
     my($class, $dir) = @_;
 
-    return 1 if is_win32; #just assume, till Apache::Build works under win32
+    return 1 if WIN32; #just assume, till Apache::Build works under win32
 
     my $instsrc;
     {
@@ -671,7 +758,7 @@ sub xsubpp {
       '$(MODPERL_PRIVLIBEXP)/ExtUtils/xsubpp',
         '-typemap', '$(MODPERL_PRIVLIBEXP)/ExtUtils/typemap';
 
-    my $typemap = $self->file_path('src/modules/perl/typemap');
+    my $typemap = $self->file_path('lib/typemap');
     if (-e $typemap) {
         $xsubpp .= join ' ',
           ' -typemap', $typemap;
@@ -691,8 +778,11 @@ sub make_xs {
     my @xs_targ;
 
     while (my($name, $xs) = each %{ $self->{XS} }) {
-        #Foo/Bar.xs => Bar.c
-        (my $c = $xs) =~ s:.*/(\w+)\.xs$:$1.c:;
+        #Foo/Bar.xs => Foo_Bar.c
+        (my $c = $xs) =~ s:.*?WrapXS/::;
+        $c =~ s:/:_:g;
+        $c =~ s:\.xs$:.c:;
+
         push @files, $c;
 
         push @xs_targ, <<EOF;
@@ -718,6 +808,7 @@ EOF
 #when we use a bit of MakeMaker, make it use our values for these vars
 my %perl_config_pm_alias = (
     PERL         => 'perlpath',
+    PERLRUN      => 'perlpath',
     PERL_LIB     => 'privlibexp',
     PERL_ARCHLIB => 'archlibexp',
 );
@@ -725,7 +816,7 @@ my %perl_config_pm_alias = (
 my $mm_replace = join '|', keys %perl_config_pm_alias;
 
 my @perl_config_pm =
-  (qw(cc cpprun rm ranlib lib_ext obj_ext cccdlflags lddlflags),
+  (qw(cc cpprun rm ranlib lib_ext obj_ext cccdlflags lddlflags optimize),
    values %perl_config_pm_alias);
 
 sub mm_replace {
@@ -733,20 +824,18 @@ sub mm_replace {
     $$val =~ s/\(($mm_replace)\)/(MODPERL_\U$perl_config_pm_alias{$1})/g;
 }
 
+#help prevent warnings
+my @mm_init_vars = (BASEEXT => '');
+
 sub make_tools {
     my($self, $fh) = @_;
 
     for (@perl_config_pm) {
         print $fh $self->canon_make_attr($_, $self->perl_config($_));
     }
-    unless ($self->{MP_DEBUG}) {
-        for (qw(optimize)) {
-            print $fh $self->canon_make_attr($_, $self->perl_config($_));
-        }
-    }
 
     require ExtUtils::MakeMaker;
-    my $mm = bless {}, 'MM';
+    my $mm = bless { @mm_init_vars }, 'MM';
     $mm->init_others;
 
     for (qw(rm_f mv ld ar cp test_f)) {
@@ -778,10 +867,14 @@ EOF
 
 sub dynamic_link_default {
     my $self = shift;
-    return $self->dynamic_link_header_default . <<'EOF';
-	-o $@
-	$(MODPERL_RANLIB) $@
-EOF
+
+    my $link = $self->dynamic_link_header_default . "\t" . '-o $@';
+
+    my $ranlib = "\t" . '$(MODPERL_RANLIB) $@';
+
+    $link .= "\n" . $ranlib unless DARWIN;
+
+    $link;
 }
 
 sub dynamic_link_MSWin32 {
@@ -836,6 +929,8 @@ sub write_src_makefile {
 
     print $fh noedit_warning_hash();
 
+    print $fh $self->canon_make_attr('makefile', basename $mf);
+
     $self->make_tools($fh);
 
     print $fh $self->canon_make_attr('ap_libs', $self->apache_libs);
@@ -843,19 +938,27 @@ sub write_src_makefile {
     print $fh $self->canon_make_attr('libname', $self->{MP_LIBNAME});
     print $fh $self->canon_make_attr('dlext', 'so'); #always use .so
 
-    print $fh $self->canon_make_attr('lib_shared',
-                       "$self->{MP_LIBNAME}.$self->{MODPERL_DLEXT}");
+    my %libs = (
+        dso    => "$self->{MP_LIBNAME}.$self->{MODPERL_DLEXT}",
+        static => "$self->{MP_LIBNAME}$self->{MODPERL_LIB_EXT}",
+    );
 
-    print $fh $self->canon_make_attr('lib_static',
-                       "$self->{MP_LIBNAME}$self->{MODPERL_LIB_EXT}");
+    #XXX short-term compat for Apache::TestConfigPerl
+    $libs{shared} = $libs{dso};
 
+    while (my($type, $lib) = each %libs) {
+        print $fh $self->canon_make_attr("lib_$type", $libs{$type});
+    }
 
+    my $libperl = join '/',
+      $self->perl_config('archlibexp'), 'CORE', $self->perl_config('libperl');
 
-    print $fh $self->canon_make_attr('libperl',
-                                     join '/',
-                                     $self->perl_config('archlibexp'),
-                                     'CORE',
-                                     $self->perl_config('libperl'));
+    #this is only used for deps, if libperl has changed, relink mod_perl.so
+    #not all perl dists put libperl where it should be, so just leave this
+    #out if it isn't in the proper place
+    if (-e $libperl) {
+        print $fh $self->canon_make_attr('libperl', $libperl);
+    }
 
     for my $method (qw(ccopts ldopts inc)) {
         print $fh $self->canon_make_attr($method, $self->$method());
@@ -865,9 +968,12 @@ sub write_src_makefile {
         print $fh $self->canon_make_attr($method, @{ $code->$method() });
     }
 
-    print $fh $self->canon_make_attr('lib', $self->is_dynamic ?
-                                     $self->{MODPERL_LIB_SHARED} :
-                                     $self->{MODPERL_LIB_STATIC});
+    my @libs;
+    for my $type (map { uc } keys %libs) {
+        push @libs, $self->{"MODPERL_LIB_$type"} if $self->{"MP_USE_$type"};
+    }
+
+    print $fh $self->canon_make_attr('lib', "@libs");
 
     for my $q (qw(LIBEXECDIR)) {
         print $fh $self->canon_make_attr("AP_$q",
@@ -890,8 +996,8 @@ all: lib
 lib: $(MODPERL_LIB)
 
 install:
-	$(MODPERL_TEST_F) $(MODPERL_LIB_SHARED) && \
-	$(MODPERL_CP) $(MODPERL_LIB_SHARED) $(MODPERL_AP_LIBEXECDIR)
+	$(MODPERL_TEST_F) $(MODPERL_LIB_DSO) && \
+	$(MODPERL_CP) $(MODPERL_LIB_DSO) $(MODPERL_AP_LIBEXECDIR)
 
 .SUFFIXES: .xs .c $(MODPERL_OBJ_EXT) .lo .i .s
 
@@ -923,12 +1029,12 @@ install:
 clean:
 	$(MODPERL_RM_F) *.a *.so *.xsc \
 	$(MODPERL_LIBNAME).exp $(MODPERL_LIBNAME).lib \
-	*$(MODPERL_OBJ_EXT) *.lo *.i *.s \
+	*$(MODPERL_OBJ_EXT) *.lo *.i *.s *.pdb \
 	$(MODPERL_CLEAN_FILES) \
 	$(MODPERL_XS_CLEAN_FILES)
 
-$(MODPERL_OBJS): $(MODPERL_H_FILES) Makefile
-$(MODPERL_PIC_OBJS): $(MODPERL_H_FILES) Makefile
+$(MODPERL_OBJS): $(MODPERL_H_FILES) $(MODPERL_MAKEFILE)
+$(MODPERL_PIC_OBJS): $(MODPERL_H_FILES) $(MODPERL_MAKEFILE)
 $(MODPERL_LIB): $(MODPERL_LIBPERL)
 
 $(MODPERL_LIBNAME)$(MODPERL_LIB_EXT): $(MODPERL_OBJS)
@@ -976,15 +1082,17 @@ sub typemaps {
 sub includes {
     my $self = shift;
     my $src  = $self->dir;
-    my $os = is_win32 ? 'win32' : 'unix';
+    my $os = WIN32 ? 'win32' : 'unix';
     my @inc = $self->file_path("src/modules/perl", "xs");
 
     push @inc, $self->mp_include_dir;
 
-    my $ainc = $self->apxs('-q' => 'INCLUDEDIR');
-    if (-d $ainc) {
-        push @inc, $ainc;
-        return \@inc;
+    unless ($self->ap_prefix_is_source_tree) {
+        my $ainc = $self->apxs('-q' => 'INCLUDEDIR');
+        if (-d $ainc) {
+            push @inc, $ainc;
+            return \@inc;
+        }
     }
 
     for ("$src/modules/perl", "$src/include",
@@ -995,13 +1103,6 @@ sub includes {
           push @inc, $_ if -d $_;
       }
 
-    my $ssl_dir = "$src/../ssl/include";
-    unless (-d $ssl_dir) {
-        my $build = $self->build_config;
-        $ssl_dir = join '/', $self->{MP_SSL_BASE} || '', 'include';
-    }
-    push @inc, $ssl_dir if -d $ssl_dir;
-
     return \@inc;
 }
 
