@@ -50,7 +50,9 @@ sub my_import {
     my $stash = \%{__PACKAGE__ . '::MY::'};
     for my $sym (keys %$stash) {
         next unless *{$stash->{$sym}}{CODE};
-        *{"MY::$sym"} = *{$stash->{$sym}}{CODE};
+        my $name = "MY::$sym";
+        undef &$name if defined &$name;
+        *$name = *{$stash->{$sym}}{CODE};
     }
 }
 
@@ -69,9 +71,11 @@ sub WriteMakefile {
     my $ccflags = $build->perl_ccopts . $build->ap_ccopts;
 
     my @opts = (
-        INC     => $inc,
-        CCFLAGS => $ccflags,
-        LIBS    => $libs,
+        INC       => $inc,
+        CCFLAGS   => $ccflags,
+        OPTIMIZE  => $build->perl_config('optimize'),
+        LDDLFLAGS => $build->perl_config('lddlflags'),
+        LIBS      => $libs,
         dynamic_lib => { OTHERLDFLAGS => $build->otherldflags },
     );
 
@@ -88,7 +92,8 @@ sub WriteMakefile {
     ExtUtils::MakeMaker::WriteMakefile(@opts, @_);
 }
 
-my %always_dynamic = map { $_, 1 } qw(ModPerl::Const Apache::Const APR::Const);
+my %always_dynamic = map { $_, 1 }
+  qw(ModPerl::Const Apache::Const APR::Const APR APR::PerlIO);
 
 sub ModPerl::MM::MY::constants {
     my $self = shift;
@@ -166,6 +171,10 @@ sub ModPerl::MM::MY::post_initialize {
 sub ModPerl::MM::MY::libscan {
     my($self, $path) = @_;
 
+    if (Apache::Build::WIN32() and $path eq 'PerlIO') {
+        return ''; #XXX: APR::PerlIO does not link on win32
+    }
+
     return '' if $path =~ m/\.(pl|cvsignore)$/;
     return '' if $path =~ m:\bCVS/:;
     return '' if $path =~ m/~$/;
