@@ -4,6 +4,7 @@ use strict;
 use warnings FATAL => 'all';
 
 use constant GvSHARED => 0; #$^V gt v5.7.0;
+use Apache::TestTrace;
 use Apache::Build ();
 use ModPerl::Code ();
 use ModPerl::TypeMap ();
@@ -130,6 +131,8 @@ EOF
                 $parms = 'items, MARK+1, SP';
             }
 
+            $thx =~ s/_ $// unless $parms;
+
             my $retval = $return_type eq 'void' ?
               ["", ""] : ["RETVAL = ", "OUTPUT:\n    RETVAL\n"];
 
@@ -153,8 +156,8 @@ sub get_value {
     if ($e->{class} eq 'PV') {
         if (my $pool = $e->{pool}) {
             $pool .= '(obj)';
-            $val = "((ST(1) == &PL_sv_undef) ? NULL :
-                    apr_pstrndup($pool, val, val_len))"
+            $val = "(SvOK(ST(1)) ?
+                    apr_pstrndup($pool, val, val_len) : NULL)"
         }
     }
 
@@ -223,11 +226,13 @@ sub prepare {
     $self->{DIR} = 'WrapXS';
     $self->{XS_DIR} = catdir fastcwd(), 'xs';
 
+    my $verbose = $Apache::TestTrace::Level eq 'debug' ? 1 : 0;
+
     if (-e $self->{DIR}) {
-        rmtree([$self->{DIR}], 1, 1);
+        rmtree([$self->{DIR}], $verbose, 1);
     }
 
-    mkpath [$self->{DIR}], 1, 0755;
+    mkpath [$self->{DIR}], $verbose, 0755;
 }
 
 sub class_dirname {
@@ -245,7 +250,10 @@ sub class_dir {
     my $dir = ($dirname =~ m:/: and $dirname !~ m:^$self->{DIR}:) ?
       catdir($self->{DIR}, $dirname) : $dirname;
 
-    mkpath [$dir], 1, 0755 unless -d $dir;
+    unless (-d $dir) {
+        mkpath [$dir], 0, 0755;
+        debug "mkdir.....$dir";
+    }
 
     $dir;
 }
@@ -272,7 +280,7 @@ sub open_class_file {
     my $name = $self->class_file($class, $file);
 
     open my $fh, '>', $name or die "open $name: $!";
-    print "writing...$name\n";
+    debug "writing...$name";
 
     return $fh;
 }
@@ -513,6 +521,7 @@ my %typemap = (
     'Apache::RequestRec' => 'T_APACHEOBJ',
     'apr_time_t' => 'T_APR_TIME',
     'APR::Table' => 'T_HASHOBJ',
+    'APR::OS::Thread' => 'T_UVOBJ',
 );
 
 sub write_typemap {
