@@ -18,6 +18,8 @@ static apr_status_t modperl_shutdown(void *data)
         modperl_xs_dl_handles_close(cdata->pool, handles);
     }
 
+    modperl_env_unload();
+
     return APR_SUCCESS;
 }
 #endif
@@ -42,7 +44,7 @@ static struct {
     apr_pool_t *p = MP_boot_data.p; \
     server_rec *s = MP_boot_data.s
 
-#if defined(USE_ITHREADS) && defined(MP_PERL_5_6_1)
+#if defined(USE_ITHREADS) && defined(MP_PERL_5_6_x)
 #   define MP_REFGEN_FIXUP
 #endif
 
@@ -96,6 +98,7 @@ static void modperl_refgen_ops_fixup(void)
 static void modperl_boot(void *data)
 {
     MP_dBOOT_DATA;
+    MP_dSCFG(s);
     dTHX; /* XXX: not too worried since this only happens at startup */
     int i;
     
@@ -115,6 +118,17 @@ static void modperl_boot(void *data)
         char *name = Perl_form(aTHX_ MP_xs_loader_name, MP_xs_loaders[i]);
         newCONSTSUB(PL_defstash, name, newSViv(1));
     }
+
+    newCONSTSUB(PL_defstash, "Apache::MPM_IS_THREADED",
+                newSViv(scfg->threaded_mpm));
+
+#ifdef MP_PERL_5_6_x
+    /* make sure DynaLoader is loaded before XSLoader
+     * to workaround bug in 5.6.1 that can trigger a segv
+     * when using modperl as a dso
+     */
+    modperl_require_module(aTHX_ "DynaLoader", FALSE);
+#endif
 }
 
 static void modperl_xs_init(pTHX)
@@ -255,6 +269,8 @@ void modperl_init(server_rec *base_server, apr_pool_t *p)
         MP_dSCFG(s);
         PerlInterpreter *perl = base_perl;
 
+        PERL_SET_CONTEXT(perl);
+
         MP_TRACE_d_do(MpSrv_dump_flags(scfg, s->server_hostname));
 
         /* if alloc flags is On, virtual host gets its own parent perl */
@@ -688,6 +704,11 @@ int modperl_response_handler_cgi(request_rec *r)
         modperl_env_request_populate(aTHX_ r);
     }
 
+    /* default is +GlobalRequest, skip if PerlOption -GlobalRequest */
+    if (MpDirGLOBAL_REQUEST(dcfg) || !MpDirSeenGLOBAL_REQUEST(dcfg)) {
+        modperl_global_request_set(r);
+    }
+
     h_stdout = modperl_io_tie_stdout(aTHX_ r);
     h_stdin  = modperl_io_tie_stdin(aTHX_ r);
 
