@@ -32,7 +32,7 @@ MP_CMD_SRV_TRACE; \
 MP_CMD_SRV_DECLARE(trace)
 {
     MP_CMD_SRV_CHECK;
-    modperl_trace_level_set(arg);
+    modperl_trace_level_set(parms->server, arg);
     return NULL;
 }
 
@@ -60,36 +60,39 @@ MP_CMD_SRV_DECLARE(requires)
     return NULL;
 }
 
-MP_CMD_SRV_DECLARE2(set_var)
+static MP_CMD_SRV_DECLARE2(handle_vars)
 {
     MP_dSCFG(parms->server);
     modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
- 
-    if (parms->path) {
-        apr_table_set(dcfg->SetVar, arg1, arg2);
-        MP_TRACE_d(MP_FUNC, "DIR: arg1 = %s, arg2 = %s\n", arg1, arg2);
-    }
-    else {
-        apr_table_set(scfg->SetVar, arg1, arg2);
-        MP_TRACE_d(MP_FUNC, "SRV: arg1 = %s, arg2 = %s\n", arg1, arg2);
+    const char *name = parms->cmd->name;
+
+    modperl_table_modify_t func =
+        strEQ(name, "PerlSetVar") ? apr_table_setn : apr_table_addn;
+
+    func(dcfg->vars, arg1, arg2);
+
+    MP_TRACE_d(MP_FUNC, "%s DIR: arg1 = %s, arg2 = %s\n",
+               name, arg1, arg2);
+
+    /* make available via Apache->server->dir_config */
+    if (!parms->path) {
+        func(scfg->vars, arg1, arg2);
+
+        MP_TRACE_d(MP_FUNC, "%s SRV: arg1 = %s, arg2 = %s\n",
+                   name, arg1, arg2);
     }
+
     return NULL;
 }
 
+MP_CMD_SRV_DECLARE2(set_var)
+{
+    return modperl_cmd_handle_vars(parms, mconfig, arg1, arg2);
+}
+
 MP_CMD_SRV_DECLARE2(add_var)
 {
-    MP_dSCFG(parms->server);
-    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
- 
-    if (parms->path) {
-        apr_table_add(dcfg->SetVar, arg1, arg2);
-        MP_TRACE_d(MP_FUNC, "DIR: arg1 = %s, arg2 = %s\n", arg1, arg2);
-    }
-    else {
-        apr_table_add(scfg->SetVar, arg1, arg2);
-        MP_TRACE_d(MP_FUNC, "SRV: arg1 = %s, arg2 = %s\n", arg1, arg2);
-    }
-    return NULL;
+    return modperl_cmd_handle_vars(parms, mconfig, arg1, arg2);
 }
 
 MP_CMD_SRV_DECLARE2(set_env)
@@ -97,6 +100,14 @@ MP_CMD_SRV_DECLARE2(set_env)
     MP_dSCFG(parms->server);
     modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
  
+#ifdef ENV_IS_CASELESS /* i.e. WIN32 */
+    /* we turn off env magic during hv_store later, so do this now,
+     * else lookups on keys with lowercase characters will fails
+     * because Perl will uppercase them prior to lookup.
+     */
+    modperl_str_toupper((char *)arg1);
+#endif
+
     MP_TRACE_d(MP_FUNC, "arg1 = %s, arg2 = %s\n", arg1, arg2);
 
     if (!parms->path) {
