@@ -39,6 +39,7 @@ modperl_interp_t *modperl_interp_new(apr_pool_t *p,
                                      modperl_interp_pool_t *mip,
                                      PerlInterpreter *perl)
 {
+    UV clone_flags = 0;
     modperl_interp_t *interp = 
         (modperl_interp_t *)apr_pcalloc(p, sizeof(*interp));
     
@@ -52,7 +53,11 @@ modperl_interp_t *modperl_interp_new(apr_pool_t *p,
         );
 #endif
 
-        interp->perl = perl_clone(perl, FALSE);
+#if defined(WIN32) && defined(CLONEf_CLONE_HOST)
+        clone_flags |= CLONEf_CLONE_HOST;
+#endif
+
+        interp->perl = perl_clone(perl, clone_flags);
 
         modperl_interp_clone_init(interp);
 
@@ -76,6 +81,8 @@ void modperl_interp_destroy(modperl_interp_t *interp)
     apr_array_header_t *handles;
     dTHXa(interp->perl);
 
+    PERL_SET_CONTEXT(interp->perl);
+
     MP_TRACE_i(MP_FUNC, "interp == 0x%lx\n",
                (unsigned long)interp);
 
@@ -152,6 +159,8 @@ apr_status_t modperl_interp_pool_destroy(void *data)
 
     mip->parent->perl = NULL;
 
+    modperl_env_unload();
+
     return APR_SUCCESS;
 }
 
