@@ -59,6 +59,9 @@ MP_INLINE SV *modperl_newSVsv_obj(pTHX_ SV *stashsv, SV *obj);
 
 MP_INLINE SV *modperl_ptr2obj(pTHX_ char *classname, void *ptr);
 
+MP_INLINE SV *modperl_perl_sv_setref_uv(pTHX_ SV *rv,
+                                        const char *classname, UV uv);
+
 apr_pool_t *modperl_sv2pool(pTHX_ SV *obj);
 
 char *modperl_apr_strerror(apr_status_t rv);
@@ -98,6 +101,8 @@ HE *modperl_perl_hv_fetch_he(pTHX_ HV *hv,
 #define hv_fetch_he(hv,k,l,h) \
     modperl_perl_hv_fetch_he(aTHX_ hv, k, l, h)
 
+void modperl_str_toupper(char *str);
+
 void modperl_perl_do_sprintf(pTHX_ SV *sv, I32 len, SV **sarg);
 
 void modperl_perl_call_list(pTHX_ AV *subs, const char *name);
