@@ -3,18 +3,21 @@ package TestAPI::uri;
 use strict;
 use warnings FATAL => 'all';
 
+use Apache::Test;
+
 use APR::URI ();
 use Apache::URI ();
+use Apache::RequestRec ();
 use Apache::RequestUtil ();
-use Apache::ServerUtil ();
-use Apache::Test;
+
+use Apache::Const -compile => 'OK';
 
 my $location = '/' . __PACKAGE__;
 
 sub handler {
     my $r = shift;
 
-    plan $r, tests => 13;
+    plan $r, tests => 14;
 
     $r->args('query');
 
@@ -27,8 +30,11 @@ sub handler {
     my $up = $uri->unparse;
     ok $up =~ m:^$location:;
 
-    my $curl = $r->construct_url($r->uri, $r->pool);
-    my $parsed = APR::URI->parse($r, $curl);
+    my $server = $r->construct_server;
+    ok $server eq join ':', $r->get_server_name, $r->get_server_port;
+
+    my $curl = $r->construct_url;
+    my $parsed = APR::URI->parse($r->pool, $curl);
 
     ok $parsed->isa('APR::URI');
 
