@@ -10,9 +10,13 @@ use warnings FATAL => 'all';
 use Apache2 ();
 
 use Apache::Test ();
-use Apache::Const -compile => 'OK';
+
+use Apache::RequestRec ();
+use Apache::RequestIO ();
 use File::Spec::Functions qw(catfile);
 
+use Apache::Const -compile => 'OK';
+
 sub handler {
     my $r = shift;
 
@@ -42,7 +46,7 @@ EOF
 1;
 __END__
 <Base>
-    PerlSwitches -Mlib=@documentroot@/testdirective/perlmodule-vh
+    PerlSwitches -I@documentroot@/testdirective/perlmodule-vh
 </Base>
 
 <VirtualHost TestDirective::perlmodule>
